@@ -273,13 +273,18 @@ class QueryParameters(object):
 
         :param mappings: Key-value pairs where the key is the client-side
                          name we'll accept here and the value is the name
-                         the server expects, e.g, changes_since=changes-since
+                         the server expects, e.g, changes_since=changes-since.
+                         Additionally, a value can be a dict with optional keys
+                         name - server-side name,
+                         type - callable to convert from client to server
+                         representation.
 
         By default, both limit and marker are included in the initial mapping
         as they're the most common query parameters used for listing resources.
         """
         self._mapping = {"limit": "limit", "marker": "marker"}
-        self._mapping.update(dict({name: name for name in names}, **mappings))
+        self._mapping.update({name: name for name in names})
+        self._mapping.update(mappings)
 
     def _validate(self, query, base_path=None):
         """Check that supplied query keys match known query mappings
@@ -290,7 +295,9 @@ class QueryParameters(object):
                           the resource.
         """
         expected_params = list(self._mapping.keys())
-        expected_params += self._mapping.values()
+        expected_params.extend(
+            value['name'] if isinstance(value, dict) else value
+            for value in self._mapping.values())
 
         if base_path:
             expected_params += utils.get_string_format_keys(base_path)
@@ -312,11 +319,25 @@ class QueryParameters(object):
                            server side name.
         """
         result = {}
-        for key, value in self._mapping.items():
-            if key in query:
-                result[value] = query[key]
-            elif value in query:
-                result[value] = query[value]
+        for client_side, server_side in self._mapping.items():
+            if isinstance(server_side, dict):
+                name = server_side['name']
+                type_ = server_side.get('type')
+            else:
+                name = server_side
+                type_ = None
+
+            if client_side in query:
+                value = query[client_side]
+            elif name in query:
+                value = query[name]
+            else:
+                continue
+
+            if type_ is not None:
+                result[name] = type_(value)
+            else:
+                result[name] = value
         return result
 
 
@@ -1206,7 +1227,7 @@ class Resource(dict):
         return self
 
     @classmethod
-    def list(cls, session, paginated=False, **params):
+    def list(cls, session, paginated=False, base_path=None, **params):
         """This method is a generator which yields resource objects.
 
         This resource object list generator handles pagination and takes query
@@ -1220,6 +1241,9 @@ class Resource(dict):
                                **When paginated is False only one
                                page of data will be returned regardless
                                of the API's support of pagination.**
+        :param str base_path: Base part of the URI for listing resources, if
+                              different from
+                              :data:`~openstack.resource.Resource.base_path`.
         :param dict params: These keyword arguments are passed through the
             :meth:`~openstack.resource.QueryParamter._transpose` method
             to find if any of them match expected query parameters to be
@@ -1241,9 +1265,11 @@ class Resource(dict):
         session = cls._get_session(session)
         microversion = cls._get_microversion_for_list(session)
 
-        cls._query_mapping._validate(params, base_path=cls.base_path)
+        if base_path is None:
+            base_path = cls.base_path
+        cls._query_mapping._validate(params, base_path=base_path)
         query_params = cls._query_mapping._transpose(params)
-        uri = cls.base_path % params
+        uri = base_path % params
 
         limit = query_params.get('limit')
 
@@ -1413,6 +1439,119 @@ class Resource(dict):
             "No %s found for %s" % (cls.__name__, name_or_id))
 
 
+class TagMixin(object):
+
+    _tag_query_parameters = {
+        'tags': 'tags',
+        'any_tags': 'tags-any',
+        'not_tags': 'not-tags',
+        'not_any_tags': 'not-tags-any',
+    }
+
+    #: A list of associated tags
+    #: *Type: list of tag strings*
+    tags = Body('tags', type=list, default=[])
+
+    def fetch_tags(self, session):
+        """Lists tags set on the entity.
+
+        :param session: The session to use for making this request.
+        :return: The list with tags attached to the entity
+        """
+        url = utils.urljoin(self.base_path, self.id, 'tags')
+        session = self._get_session(session)
+        response = session.get(url)
+        exceptions.raise_from_response(response)
+        # NOTE(gtema): since this is a common method
+        # we can't rely on the resource_key, because tags are returned
+        # without resource_key. Do parse response here
+        json = response.json()
+        if 'tags' in json:
+            self._body.attributes.update({'tags': json['tags']})
+        return self
+
+    def set_tags(self, session, tags=[]):
+        """Sets/Replaces all tags on the resource.
+
+        :param session: The session to use for making this request.
+        :param list tags: List with tags to be set on the resource
+        """
+        url = utils.urljoin(self.base_path, self.id, 'tags')
+        session = self._get_session(session)
+        response = session.put(url, json={'tags': tags})
+        exceptions.raise_from_response(response)
+        self._body.attributes.update({'tags': tags})
+        return self
+
+    def remove_all_tags(self, session):
+        """Removes all tags on the entity.
+
+        :param session: The session to use for making this request.
+        """
+        url = utils.urljoin(self.base_path, self.id, 'tags')
+        session = self._get_session(session)
+        response = session.delete(url)
+        exceptions.raise_from_response(response)
+        self._body.attributes.update({'tags': []})
+        return self
+
+    def check_tag(self, session, tag):
+        """Checks if tag exists on the entity.
+
+        If the tag does not exist a 404 will be returned
+
+        :param session: The session to use for making this request.
+        :param tag: The tag as a string.
+        """
+        url = utils.urljoin(self.base_path, self.id, 'tags', tag)
+        session = self._get_session(session)
+        response = session.get(url)
+        exceptions.raise_from_response(response,
+                                       error_message='Tag does not exist')
+        return self
+
+    def add_tag(self, session, tag):
+        """Adds a single tag to the resource.
+
+        :param session: The session to use for making this request.
+        :param tag: The tag as a string.
+        """
+        url = utils.urljoin(self.base_path, self.id, 'tags', tag)
+        session = self._get_session(session)
+        response = session.put(url)
+        exceptions.raise_from_response(response)
+        # we do not want to update tags directly
+        tags = self.tags
+        tags.append(tag)
+        self._body.attributes.update({
+            'tags': tags
+        })
+        return self
+
+    def remove_tag(self, session, tag):
+        """Removes a single tag from the specified server.
+
+        :param session: The session to use for making this request.
+        :param tag: The tag as a string.
+        """
+        url = utils.urljoin(self.base_path, self.id, 'tags', tag)
+        session = self._get_session(session)
+        response = session.delete(url)
+        exceptions.raise_from_response(response)
+        # we do not want to update tags directly
+        tags = self.tags
+        try:
+            # NOTE(gtema): if tags were not fetched, but request suceeded
+            # it is ok. Just ensure tag does not exist locally
+            tags.remove(tag)
+        except ValueError:
+            pass  # do nothing!
+        self._body.attributes.update({
+            'tags': tags
+        })
+        return self
+
+
 def _normalize_status(status):
     if status is not None:
         status = status.lower()
